#!/usr/bin/env ruby

require 'studio_game/player'
require 'studio_game/GameTurn'
require 'studio_game/treasure_trove'

module StudioGame
    describe Player do
        before do
            # stdout is a global variable, when overwritten
            # it's writes out to the new StringIO object 
            # and not the console
            #$stdout = StringIO.new
            @p1_health, @p2_health = 100, 100
            @player = Player.new("nick", @p1_health)
            @player2 = Player.new("jeff", @p2_health)
        end

        it 'has a captialized name' do
            expect(@player.name).to eq("Nick")
        end

        it 'has an initial health' do
            expect(@player.health).to eq(100)
        end

        it 'has a string representation' do
            @player.found_treasure(Treasure.new(:lightning, 5))
            expect(@player.to_s).to eq("I'm Nick with a health of 100 and a score of 105")
        end

        it 'computers a score as the sum of its health and points' do
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.found_treasure(Treasure.new(:warchest, 50000))
            expect(@player.score).to eq(50150) 
        end
        it 'increase health by 15 when w00ted' do
            @player.w00t
            expect(@player.health).to eq(115)
        end

        it 'decreases health by 10 when blammed' do
            @player.blam
            expect(@player.health).to eq(90)
        end

        it 'checks random damage by using special attack' do
            allow_any_instance_of(Player).to receive(:random_score).and_return(50)
            @player.use_special_attack(@player2)
            expect(@player2.health).to eq(@p2_health - 50)
        end

        it 'computes points as the sum of all treasure points' do 
            expect(@player.points).to eq(0)
            @player.found_treasure(Treasure.new(:hammer, 50))
            expect(@player.points).to eq(50)
            @player.found_treasure(Treasure.new(:bazooka, 1000))
            expect(@player.points).to eq(1050)
            @player.found_treasure(Treasure.new(:machine_learning, 5000))
            expect(@player.points).to eq(6050)
        end

        it 'yields each found treasure and its total points' do
            @player.found_treasure(Treasure.new(:skillet, 100))
            @player.found_treasure(Treasure.new(:skillet, 100))
            @player.found_treasure(Treasure.new(:hammer, 50))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
            @player.found_treasure(Treasure.new(:bottle, 5))
        
            yielded = []
            @player.each_found_treasure do |treasure|
            yielded << treasure
            end
        
            expect(yielded).to eq([
            Treasure.new(:skillet, 200),
            Treasure.new(:hammer, 50),
            Treasure.new(:bottle, 25)
        ])
        end

        context 'created with a default score' do
            before do
                @player = Player.new("nick", 100, 50)
            end

            it 'has a score of 50' do
                expect(@player.score).to eq(100)
            end
        end

        context 'created with a health of 150' do
            before do 
                @player = Player.new("jeff", 150)
            end

            it 'is a strong player' do
                expect(@player.strong?).to eq(true)
                expect(@player.strong?).to_not eq(false)
            end
        end

        context 'created with health of 75' do 
            before do
                @player = Player.new("tim", 75)
            end

            it 'is a weak player' do 
                expect(@player.weak?).to eq(true)
                expect(@player.weak?).to_not eq(false)
            end
        end

        context 'multiple players' do
            before do 
                @player1 = Player.new("nick", 100)
                @player2 = Player.new("jeff", 200)
                @player3 = Player.new("matt", 300)
                @players = [@player1, @player2, @player3]
            end
            
            it 'names should be sorted by decreasing score' do 
                expect(@players.sort).to eq([@player3, @player2, @player1])
            end
        end
    end
end